@@ -4,14 +4,14 @@ var N = 40;
 var finished = false;
 
 function spawn (i) {
-  var p = node.createProcess('python -c "print 500 * 1024 * \'C\'"'); 
+  var child = node.createChildProcess('python -c "print 500 * 1024 * \'C\'"');
   var output = "";
 
-  p.addListener("output", function(chunk) { 
+  child.addListener("output", function(chunk) {
     if (chunk) output += chunk;
-  }); 
+  });
 
-  p.addListener("exit", function () {
+  child.addListener("exit", function () {
     //puts(output);
     if (i < N)
       spawn(i+1);
@@ -20,10 +20,8 @@ function spawn (i) {
   });
 }
 
-function onLoad () {
-  spawn(0);
-}
+spawn(0);
 
-function onExit () {
+process.addListener("exit", function () {
   assertTrue(finished);
-}
+});
